# coding: latin1
 
'''Module permettant l'interpolation des caractristiques principales d'une 
toile ( savoir sa luminosit et sa temprature de surface) connaissant sa 
masse m et son ge t. Bas sur des simulation EZ 
(cf http://www.astro.wisc.edu/~townsend/static.php?ref=ez-web)

ATTENTION: l'intervalle de masse ne s'tend que de 0.1 masse solaire  25 
masses solaires. '''

import glob
import numpy as np


# Les fichiers contenant les donnes pour chaque toile
_files = glob.glob('EZ_*.txt')

# Quelques variables de stockage
_masses = []                    # Les masses des toiles
_temps  = []                    # Leur dernier souffle de vie
_EZ_exacts_Linterpolations = [] # Interpolations en luminosit
_EZ_exacts_Tinterpolations = [] # Interpolations en temprature

import scipy.interpolate        # Pour les interpolations

for _file in _files:            # On boucle sur les fichiers de donnes
    # La masse est code dans le nom du fichier
    _masse = float(_file.replace('EZ_','').replace('.txt',''))
    _data = np.loadtxt(_file)   # Lecture des donnes
    _times= _data[:,0]/1e6      # La premire est l'instant t
    _L_val= _data[:,1]          # Puis log10(L)
    _T_val= _data[:,2]          # et enfin log10(T)
    # On stocke les fonctions interpolatrices
    _EZ_exacts_Linterpolations.append(scipy.interpolate.interp1d(_times,_L_val))
    _EZ_exacts_Tinterpolations.append(scipy.interpolate.interp1d(_times,_T_val))
    _masses.append(_masse)      # ainsi que les masses
    _temps.append(_times[-1])   # et leur dernier souffle de vie.
    
# Pour vrifier si jamais la loi fin_de_vie(m) est bien monotone
#import matplotlib.pyplot as plt
#
#plt.loglog(_masses,_temps)
#plt.show()


# Fonction pour interpoler le dernier souffle de vie
_fin_de_vie = scipy.interpolate.interp1d(_masses,_temps)
# et la fonction rciproque qui donne la masse maximale vivante  un ge donn.
# EDIT: n'est plus d'actualit, cf plus bas
#_masse_maximale = scipy.interpolate.interp1d(_temps,_masses)

def _getL(m,t):
    # On trouve les masses encadrant la masse qui nous intresse
    for i in range(len(_masses)):
        if _masses[i] > m: break
    # Si jamais l'toile suprieure est morte, on prend la dernire luminosit connue
    if _temps[i] < t: Li =  float(_EZ_exacts_Linterpolations[i](_temps[i]))
    # sinon on estime sa luminosit  l'instant t
    else: Li = float(_EZ_exacts_Linterpolations[i](t))
    # On estime le pourcentage de proximit  l'toile suprieure
    x = (m - _masses[i-1])/(_masses[i] - _masses[i-1])
    # et on fait une interpolation entre les deux valeurs de luminosit
    return x*Li + (1-x)*float(_EZ_exacts_Linterpolations[i-1](t))

def _getT(m,t):
    # On trouve les masses encadrant la masse qui nous intresse
    for i in range(len(_masses)):
        if _masses[i] > m: break
    # Si jamais l'toile suprieure est morte, on prend la dernire temprature connue
    if _temps[i] < t: 
        Ti = float(_EZ_exacts_Tinterpolations[i](_temps[i]))
    # sinon on estime sa temprature  l'instant t
    else: 
        Ti = float(_EZ_exacts_Tinterpolations[i](t))
    # On estime le pourcentage de proximit  l'toile suprieure
    x = (m - _masses[i-1])/(_masses[i] - _masses[i-1])
    # et on fait une interpolation entre les deux valeurs de luminosit
    return x*Ti + (1-x)*float(_EZ_exacts_Tinterpolations[i-1](t))

def fin_de_vie(m):
    '''Renvoie la valeur (en million d'annes) du temps de fin de vie estim 
    pour une toile de masse initiale m (exprime en masses solaires).'''
    return float(_fin_de_vie(m))

def masse_maximale(t):
    '''Renvoie la valeur (en masses solaires) de la masse de la plus grosse 
    toile encore en vie au temps t (exprim en millions d'annes) parmi 
    celles disponibles.'''
    if t < _temps[-1]: return _masses[-1]
    for i in range(len(_temps)-1,-1,-1):        # On regarde en commenant par les plus massives
        if _temps[i] > t: break                 # laquelle est encore vivante
    x = (t - _temps[i])/(_temps[i+1]-_temps[i]) # et on interpole.
    return x*_masses[i+1] + (1-x)*_masses[i]

def getL(m,t):
    '''Renvoie le logarithme dcimal de la luminosit (mesure en luminosits 
    solaires) d'une toile de masse initiale m (exprime en masses solaires)  
    une date t (en millions d'annes). 
    Renvoie None si jamais l'toile n'est plus de ce monde (RIP).'''
    if t > fin_de_vie(m): return None
    else: return float(_getL(m,t))

def getT(m,t):
    '''Renvoie le logarithme dcimal de la temprature de surface (exprime en 
    kelvins) d'une toile de masse initiale m (exprime en masses solaires)  
    une date t (en millions d'annes).
    Renvoie None si jamais l'toile n'est plus de ce monde (RIP).'''
    if t > fin_de_vie(m): return None
    else: return float(_getT(m,t))

